/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2010-2011 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * http://glassfish.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.jersey.server.impl.uri;

import org.junit.*;
import static org.junit.Assert.*;
import java.net.URI;

/**
 * @author Yegor Bugayenko (yegor256@java.net)
 */
public class UriHelperTest {

    @Test
    public void testDotSegmentsRemovalAlgorithm() throws Exception {
        this.validateRemoveDotSegments(null, true, null);
        this.validateRemoveDotSegments("", true, "");
        this.validateRemoveDotSegments("abc/cde/efg", true, "abc/cde/efg");
        this.validateRemoveDotSegments("/abc/cde/efg/../test", true, "/abc/cde/test");
        this.validateRemoveDotSegments("/abc/./why/not/../test", true, "/abc/why/test");
        this.validateRemoveDotSegments("/very/long/list/of/segments/../../../../../test///", false, "/test/");
        this.validateRemoveDotSegments("/some/../slashes///", true, "/slashes///");
        this.validateRemoveDotSegments("../some/path", true, "some/path");
        this.validateRemoveDotSegments("./some/path", true, "some/path");
        this.validateRemoveDotSegments("some/..", true, "/");
    }

    private void validateRemoveDotSegments(String path,
        boolean preserveContdSlashes, String expected) throws Exception {
        String result = UriHelper.removeDotSegments(path, preserveContdSlashes);
        assertEquals(
            String.format(
                "removeDotSegments('%s', %s) returned '%s' while '%s' expected",
                path,
                preserveContdSlashes ? "TRUE" : "FALSE",
                result,
                expected
            ),
            expected,
            result
        );
    }

    @Test
    public void testNormalizesURIs() throws Exception {
        this.validateNormalize("/some-path", true, "/some-path");
        this.validateNormalize("http://example.com/some/../path", true, "http://example.com/path");
        this.validateNormalize("http://example.com/../path", true, "http://example.com/../path");
        this.validateNormalize("http://example.com//path", true, "http://example.com//path");
    }

    private void validateNormalize(String path,
        boolean preserveContdSlashes, String expected) throws Exception {
        URI uri = new URI(path);
        URI result = UriHelper.normalize(uri, preserveContdSlashes);
        assertEquals(
            String.format(
                "normalize('%s', %s) returned '%s' while '%s' expected",
                uri.toString(),
                preserveContdSlashes ? "TRUE" : "FALSE",
                result.toString(),
                expected
            ),
            expected,
            result.toString()
        );
    }

}