/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2010-2013 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * http://glassfish.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.jersey.impl.methodparams;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.URI;
import java.util.List;

import javax.ws.rs.DefaultValue;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;

import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.impl.AbstractResourceTester;

/**
 * @author Paul Sandoz
 */
public class QueryParamStringConstructorTest extends AbstractResourceTester {

    public QueryParamStringConstructorTest(String testName) {
        super(testName);
    }

    @Path("/")
    public static class ResourceString {
        @GET
        public String doGet(
                @QueryParam("arg1") BigDecimal arg1,
                @QueryParam("arg2") BigInteger arg2,
                @QueryParam("arg3") URI arg3) {
            assertEquals("3.145", arg1.toString());
            assertEquals("3145", arg2.toString());
            assertEquals("http://test", arg3.toString());
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringList {
        @GET
        public String doGetString(@QueryParam("args") List<BigDecimal> args) {
            assertEquals("3.145", args.get(0).toString());
            assertEquals("2.718", args.get(1).toString());
            assertEquals("1.618", args.get(2).toString());
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringListEmpty {
        @GET
        public String doGetString(@QueryParam("args") List<BigDecimal> args) {
            assertEquals(3, args.size());
            assertEquals(null, args.get(0));
            assertEquals(null, args.get(1));
            assertEquals(null, args.get(2));
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringNull {
        @GET
        public String doGet(
                @QueryParam("arg1") BigDecimal arg1,
                @QueryParam("arg2") BigInteger arg2) {
            assertEquals(null, arg1);
            assertEquals(null, arg2);
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringNullDefault {
        @GET
        public String doGet(
                @QueryParam("arg1") BigDecimal arg1) {
            assertEquals(null, arg1);
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringDefault {
        @GET
        public String doGet(
                @QueryParam("arg1") @DefaultValue("3.145") BigDecimal arg1) {
            assertEquals("3.145", arg1.toString());
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringDefaultOverride {
        @GET
        public String doGet(
                @QueryParam("arg1") @DefaultValue("3.145") BigDecimal arg1) {
            assertEquals("2.718", arg1.toString());
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringListEmptyDefault {
        @GET
        public String doGetString(@QueryParam("args") List<BigDecimal> args) {
            assertEquals(0, args.size());
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringListDefault {
        @GET
        public String doGetString(
                @QueryParam("args") @DefaultValue("3.145") List<BigDecimal> args) {
            assertEquals("3.145", args.get(0).toString());
            return "content";
        }
    }

    @Path("/")
    public static class ResourceStringListDefaultOverride {
        @GET
        public String doGetString(
                @QueryParam("args") @DefaultValue("3.145") List<BigDecimal> args) {
            assertEquals("2.718", args.get(0).toString());
            return "content";
        }
    }

    public void testStringConstructorGet() {
        initiateWebApplication(ResourceString.class);

        resource("/?arg1=3.145&arg2=3145&arg3=http:%2F%2Ftest").
                get(String.class);
    }

    public void testStringConstructorListGet() {
        initiateWebApplication(ResourceStringList.class);

        resource("/?args=3.145&args=2.718&args=1.618").
                accept("application/stringlist").
                get(String.class);
    }

    public void testStringConstructorListEmptyGet() {
        initiateWebApplication(ResourceStringListEmpty.class);

        resource("/?args&args&args").
                accept("application/stringlist").
                get(String.class);
    }

    public void testStringConstructorNullDefault() {
        initiateWebApplication(ResourceStringNullDefault.class);

        resource("/").get(String.class);
    }

    public void testStringConstructorDefault() {
        initiateWebApplication(ResourceStringDefault.class);

        resource("/").get(String.class);
    }

    public void testStringConstructorDefaultOverride() {
        initiateWebApplication(ResourceStringDefault.class);

        resource("/?args=2.718").
                get(String.class);
    }

    public void testStringConstructorListEmptyDefault() {
        initiateWebApplication(ResourceStringListEmptyDefault.class);

        resource("/").get(String.class);
    }

    public void testStringConstructorListDefault() {
        initiateWebApplication(ResourceStringListDefault.class);

        resource("/").get(String.class);
    }

    public void testStringConstructorListDefaultOverride() {
        initiateWebApplication(ResourceStringListDefaultOverride.class);

        resource("/?args=2.718").
                get(String.class);
    }

    public void testStringConstructorNullGet() {
        initiateWebApplication(ResourceStringNull.class);

        resource("/?arg1=&arg2=").get(String.class);
    }

    public void testBadStringConstructorValue() {
        initiateWebApplication(ResourceString.class);

        ClientResponse response = resource("/?arg1=ABCDEF&arg2=3145&arg3=http:%2F%2Ftest", false).
                get(ClientResponse.class);
        assertEquals(404, response.getStatus());
    }
}
